@@ -2,11 +2,9 @@
 import gettext
 import gi
 import locale
-import magic
 import os
 import re
 import setproctitle
-import subprocess
 import warnings
 import sys
 import functools
@@ -33,6 +31,10 @@ SCOPE_NAME_ONLY = "name"
 SCOPE_EXTENSION_ONLY = "extension"
 SCOPE_ALL = "all"
 
+SETTINGS_SCHEMA_ID = "org.x.bulky"
+MRU_OPERATION = "mru-operation"
+MRU_SCOPE = "mru-scope"
+
 class FolderFileChooserDialog(Gtk.Dialog):
     def __init__(self, window_title, transient_parent, starting_location):
         super(FolderFileChooserDialog, self).__init__(title=window_title,
@@ -163,7 +165,10 @@ class FileObject():
             return parent.get_basename()
 
     def writable(self):
-        return self.info.get_attribute_boolean("access::can-write")
+        if self.gfile.is_native():
+            return self.info.get_attribute_boolean("access::can-write")
+        # For non-native (remote) files, optimistically assume writable
+        return True
 
     def parent_writable(self):
         parent = self.gfile.get_parent()
@@ -297,12 +302,22 @@ class MainWindow():
         self.treeview.get_selection().connect("changed", self.on_files_selected)
 
         # Combos
-        self.builder.get_object("combo_operation").connect("changed", self.on_operation_changed)
-        self.builder.get_object("combo_scope").connect("changed", self.on_scope_changed)
+        self.combo_operation = self.builder.get_object("combo_operation")
+        self.combo_operation.set_active_id(self.settings.get_string(MRU_OPERATION))
+        self.combo_operation.connect("changed", self.on_operation_changed)
+        self.combo_scope = self.builder.get_object("combo_scope")
+        self.combo_scope.set_active_id(self.settings.get_string(MRU_SCOPE))
+        self.combo_scope.connect("changed", self.on_scope_changed)
+
         self.stack = self.builder.get_object("stack")
         self.infobar = self.builder.get_object("infobar")
         self.error_label = self.builder.get_object("error_label")
 
+        # Ini "MRU"-view
+        self.on_operation_changed(self.combo_operation)
+        self.on_scope_changed(self.combo_scope)
+
+
         # Replace widgets
         self.find_entry = self.builder.get_object("find_entry")
         self.replace_entry = self.builder.get_object("replace_entry")
@@ -313,6 +328,19 @@ class MainWindow():
         self.replace_regex_check.connect("toggled", self.on_widget_change)
         self.replace_case_check.connect("toggled", self.on_widget_change)
 
+        self.replace_start_spin = self.builder.get_object("replace_start_spin")
+        self.replace_start_spin.connect("value-changed", self.on_widget_change)
+        self.replace_start_spin.set_range(0, 10000)
+        self.replace_start_spin.set_value(1)
+        self.replace_start_spin.set_increments(1, 10)
+
+        self.replace_inc_spin = self.builder.get_object("replace_inc_spin")
+        self.replace_inc_spin.connect("value-changed", self.on_widget_change)
+        self.replace_inc_spin.set_range(1, 1000)
+        self.replace_inc_spin.set_value(1)
+        self.replace_inc_spin.set_increments(1, 10)
+
+
         # Set focus chain
         # Not that this is deprecated (but not implemented differently) in Gtk3.
         # If we move to GTK4, we'll just drop this line of code.
@@ -328,9 +356,9 @@ class MainWindow():
         self.remove_from_check.connect("toggled", self.on_widget_change)
         self.remove_to_check.connect("toggled", self.on_widget_change)
         self.remove_from_spin.set_range(1, 100)
-        self.remove_from_spin.set_increments(1, 1)
+        self.remove_from_spin.set_increments(1, 10)
         self.remove_to_spin.set_range(1, 100)
-        self.remove_to_spin.set_increments(1, 1)
+        self.remove_to_spin.set_increments(1, 10)
 
         # Insert widgets
         self.insert_entry = self.builder.get_object("insert_entry")
@@ -342,7 +370,20 @@ class MainWindow():
         self.insert_reverse_check.connect("toggled", self.on_widget_change)
         self.overwrite_check.connect("toggled", self.on_widget_change)
         self.insert_spin.set_range(1, 100)
-        self.insert_spin.set_increments(1, 1)
+        self.insert_spin.set_increments(1, 10)
+
+        self.insert_start_spin = self.builder.get_object("insert_start_spin")
+        self.insert_start_spin.connect("value-changed", self.on_widget_change)
+        self.insert_start_spin.set_range(0, 10000)
+        self.insert_start_spin.set_value(1)
+        self.insert_start_spin.set_increments(1, 10)
+
+        self.insert_inc_spin = self.builder.get_object("insert_inc_spin")
+        self.insert_inc_spin.connect("value-changed", self.on_widget_change)
+        self.insert_inc_spin.set_range(1, 1000)
+        self.insert_inc_spin.set_value(1)
+        self.insert_inc_spin.set_increments(1, 10)
+
 
         # Case widgets
         self.radio_titlecase = self.builder.get_object("radio_titlecase")
@@ -358,7 +399,7 @@ class MainWindow():
         self.radio_accents.connect("toggled", self.on_widget_change)
 
         # Tooltips
-        variables_tooltip = _("Use %n, %0n, %00n, %000n to enumerate.")
+        variables_tooltip = _("Use %n, %0n, %00n, %000n, etc. to enumerate.")
         self.replace_entry.set_tooltip_text(variables_tooltip)
         self.insert_entry.set_tooltip_text(variables_tooltip)
 
@@ -397,17 +438,13 @@ class MainWindow():
         dlg.set_program_name("Bulky")
         dlg.set_comments(_("Rename Files"))
         try:
-            h = open('/usr/share/licenses/spdx/GPL-3.0-or-later.txt', encoding="utf-8")
-            s = h.readlines()
-            gpl = ""
-            for line in s:
-                gpl += line
-            h.close()
+            with open('/usr/share/common-licenses/GPL', encoding="utf-8") as h:
+                gpl= h.read()
             dlg.set_license(gpl)
         except Exception as e:
             print (e)
 
-        dlg.set_version("3.6")
+        dlg.set_version("3.8")
         dlg.set_icon_name("bulky")
         dlg.set_logo_icon_name("bulky")
         dlg.set_website("https://www.github.com/linuxmint/bulky")
@@ -443,6 +480,7 @@ class MainWindow():
             file_uri = self.model.get_value(iter, COL_FILE).uri
             self.uris.remove(file_uri)
             self.model.remove(iter)
+        self.treeview.columns_autosize()
         self.preview_changes()
 
     def on_add_button(self, widget):
@@ -477,6 +515,10 @@ class MainWindow():
         if Gio.IOErrorEnum(error.code) == Gio.IOErrorEnum.FILENAME_TOO_LONG:
             message = _("Unable to rename '%s': File name too long") \
                 % file_obj.get_path_or_uri_for_display()
+        elif not file_obj.gfile.is_native() and error.code == 0:
+            message = _("Unable to rename '%s': Remote operation failed. " \
+                "This may be due to insufficient permissions or a server error.") \
+                % file_obj.get_path_or_uri_for_display()
         else:
             message = _("Unable to rename '%s': %s") \
                 % (file_obj.get_path_or_uri_for_display(), error.message)
@@ -588,13 +630,21 @@ class MainWindow():
         elif operation_id == "case":
             self.stack.set_visible_child_name("case_page")
             self.operation_function = self.change_case
+
+        self.settings.set_string(MRU_OPERATION, operation_id)
         self.preview_changes()
 
     def on_scope_changed(self, widget):
         self.scope = widget.get_active_id()
+
+        self.settings.set_string(MRU_SCOPE, self.scope)
         self.preview_changes()
 
     def on_widget_change(self, widget):
+        if self.replace_regex_check.get_active():
+            self.find_entry.set_placeholder_text("Enter a regular expression; example: .+")
+        else:
+            self.find_entry.set_placeholder_text("Enter a search string; wildcards ? and * are supported.")
         self.preview_changes()
 
     def preview_changes(self):
@@ -673,8 +723,12 @@ class MainWindow():
         case = self.replace_case_check.get_active()
         regex = self.replace_regex_check.get_active()
         find = self.find_entry.get_text()
+        if not find:  #ignore empty search string
+            return string
+        inc  = self.replace_inc_spin.get_value_as_int()
+        start= self.replace_start_spin.get_value_as_int()
         replace = self.replace_entry.get_text()
-        replace = self.inject(index, replace)
+        replace = self.inject((index-1)*inc + start, replace)
         try:
             if regex:
                 if case:
@@ -710,8 +764,9 @@ class MainWindow():
 
     def insert_text(self, index, string):
         text = self.insert_entry.get_text()
-        text = self.inject(index, text)
-        length = len(string) - 1
+        inc  = self.insert_inc_spin.get_value_as_int()
+        start= self.insert_start_spin.get_value_as_int()
+        text = self.inject((index-1)*inc + start, text)
         from_index = self.insert_spin.get_value_as_int() - 1
         a = len(string)
         b = len(text)
@@ -740,11 +795,11 @@ class MainWindow():
             return unidecode.unidecode(string)
 
     def inject(self, index, string):
-        string = string.replace('%n', "{:01d}".format(index))
-        string = string.replace('%0n', "{:02d}".format(index))
-        string = string.replace('%00n', "{:03d}".format(index))
-        string = string.replace('%000n', "{:04d}".format(index))
-        return string
+        def repl(match):
+            zeros = match.group(1)
+            width = len(zeros) + 1
+            return f"{index:0{width}d}"
+        return re.sub(r'%([0]*)n', repl, string)
 
 '''
 TODO
